import { Component, TemplateRef, ViewChild } from '@angular/core';
import { AgencyService } from '../services/agency-rates.service';
import { getCacheData } from '../utils/helper';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { FormGroup, FormControl, Validators } from '@angular/forms';

@Component({
  selector: 'app-agency-rates',
  templateUrl: './agency-rates.component.html',
  styleUrls: ['./agency-rates.component.css'],
})
export class AgencyRatesComponent {
  @ViewChild('manageAgencyRates') manageAgencyRatesModal;
  bsModalRef: BsModalRef;

  type: any = 'add';

  selectedCurrencyType = 'All';

  currencyTypes: any = [
    {
      id: 2,
      name: 'Sending',
    },
    {
      id: 3,
      name: 'Receiving',
    },
    {
      id: 4,
      name: 'Send/Receive',
    },
  ];

  currencies: any = [];
  filterCurrencies: any = [];
  selectedCurrency: any = 118;
  settlementCurrencies: any = [];
  agencyRates: any = [];

  manageAgencyRatesForm = new FormGroup({
    SettlementCurrency: new FormControl('None', Validators.required),
    Currency_ID: new FormControl('None', Validators.required),
    Bid_Rate: new FormControl('', Validators.required),
    Offer_Rate: new FormControl('', Validators.required),
    Revalution_Rate: new FormControl('', Validators.required),
    Settelment_Rate: new FormControl('', Validators.required),
    AutoRateflag: new FormControl(false),
    ExecutionTime: new FormControl(0),
  });

  constructor(
    private agencyService: AgencyService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getAllCurrencies();
    this.getSettlementCurrencies();
  }

  getAgencyRates(currencyId: any = 0) {
    this.agencyService
      .getAgencyRates(
        getCacheData('admin_token'),
        getCacheData('username'),
        currencyId
      )
      .subscribe({
        next: (res: any) => {
          this.agencyRates = res;
          for (let index = 0; index < this.agencyRates.length; index++) {
            const element = this.agencyRates[index];
            if (
              element?.BidRateUSD !== null &&
              element?.BidRateUSD !== undefined
            ) {
              element.BidRateUSD = this.formatWithSixDecimalPlaces(
                element.BidRateUSD
              );
            }
            if (
              element?.OfferRateUSD !== null &&
              element?.OfferRateUSD !== undefined
            ) {
              element.OfferRateUSD = this.formatWithSixDecimalPlaces(
                element.OfferRateUSD
              );
            }
            if (
              element?.RevaluationRateUSD !== null &&
              element?.RevaluationRateUSD !== undefined
            ) {
              element.RevaluationRateUSD = this.formatWithSixDecimalPlaces(
                element.RevaluationRateUSD
              );
            }
            if (
              element?.SettelmentRateUSD !== null &&
              element?.SettelmentRateUSD !== undefined
            ) {
              element.SettelmentRateUSD = this.formatWithSixDecimalPlaces(
                element.SettelmentRateUSD
              );
            }
          }
        },
        error: (err) => {
          this.agencyRates = [];
        },
        complete: () => {},
      });
  }
  // Add this method to your component
  formatWithSixDecimalPlaces(value: number | string): string {
    if (value === null || value === undefined) {
      return '';
    }

    const stringValue = value.toString();
    const [integerPart, fractionalPart] = stringValue.split('.');

    if (fractionalPart) {
      // Pad with zeros to ensure six decimal places
      return `${integerPart}.${fractionalPart.padEnd(6, '0').slice(0, 6)}`;
    } else {
      // If no decimal places, add six zeros
      return `${integerPart}.000000`;
    }
  }

  getAllCurrencies(currencyType: any = 0) {
    this.agencyService
      .getCurrencies(
        getCacheData('admin_token'),
        getCacheData('username'),
        currencyType
      )
      .subscribe({
        next: (res: any) => {
          this.currencies = res;
        },
        error: (err) => {
          this.currencies = [];
        },
        complete: () => {},
      });
  }

  onCurrencyTypeChange() {
    if (this.selectedCurrencyType != 'All') {
      this.getAllCurrencies(this.selectedCurrencyType);
    } else {
      this.getAllCurrencies();
    }
  }

  onCurrencyChange() {
    this.agencyRates = [];
    this.getAgencyRates(this.selectedCurrency);
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openAgencyRatesModal(type: any) {
    if (type == 'add') {
      this.manageAgencyRatesForm.reset({
        SettlementCurrency: 'None',
        Currency_ID: 'None',
      });
    }
    this.type = type;
    this.openModal(this.manageAgencyRatesModal);
  }

  getSettlementCurrencies() {
    this.agencyService
      .getSettlementCurrencies(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.settlementCurrencies = res;
        },
        error: (err) => {
          this.settlementCurrencies = [];
        },
        complete: () => {},
      });
  }

  get f() {
    return this.manageAgencyRatesForm.value;
  }

  addAgencyRates() {
    if (!this.manageAgencyRatesForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    if (this.f.SettlementCurrency == 'None') {
      alert('Please Select Settlement Currency!');
      return;
    }

    if (this.f.Currency_ID == 'None') {
      alert('Please Select Currency!');
      return;
    }

    if (this.f.Offer_Rate < this.f.Bid_Rate) {
      alert(
        'Offer Rate [USD] should be greater than or equal to Bid Rate [USD]!'
      );
      return;
    }

    if (this.f.Settelment_Rate < this.f.Bid_Rate) {
      alert(
        'Settlement Rate [USD] should be greater than or equal to Bid Rate [USD]!'
      );
      return;
    }

    if (this.type == 'edit' && this.f.ExecutionTime <= 14) {
      alert('Execution Time should be more than 14!');
      return;
    }

    let payload;
    payload = {
      SettlementCurrency: this.f.SettlementCurrency,
      Currency_ID: this.f.Currency_ID,
      Bid_Rate: Number(this.f.Bid_Rate),
      Offer_Rate: Number(this.f.Offer_Rate),
      Revalution_Rate: Number(this.f.Revalution_Rate),
      Settelment_Rate: Number(this.f.Settelment_Rate),
      ExecutionTime: 0,
      AutoRateflag: true,
      DtEntered: new Date(),
      TmEntered: new Date(),
      EnteredBy: '1001000',
      Message: 'none',
    };

    if (this.type === 'edit') {
      payload = [
        {
          SettlementCurrency: this.f.SettlementCurrency,
          Currency_ID: this.f.Currency_ID,
          Bid_Rate: Number(this.f.Bid_Rate),
          Offer_Rate: Number(this.f.Offer_Rate),
          Revalution_Rate: Number(this.f.Revalution_Rate),
          Settelment_Rate: Number(this.f.Settelment_Rate),
          ExecutionTime: Number(this.f.ExecutionTime),
          AutoRateflag: this.f.AutoRateflag,
          DtEntered: new Date(),
          TmEntered: new Date(),
          EnteredBy: '1001000',
          Message: 'none',
        },
      ];
    }

    const serviceMethod =
      this.type === 'add' ? 'addAgencyRates' : 'updateAgencyRates';

    this.agencyService[serviceMethod](
      getCacheData('admin_token'),
      getCacheData('username'),
      payload
    ).subscribe({
      next: (res: any) => {
        this.getAgencyRates(this.selectedCurrency);
        this.modalService.hide();
      },
      error: (err) => {
        this.modalService.hide();
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  onSettlementCurrencyChange(currencyType: any) {
    // this.getAllCurrencies(currencyType);
  }

  closeModal() {
    this.selectedCurrency = 118;
    this.getAllCurrencies();
    this.bsModalRef.hide();
  }

  editAgencyRates(agencyRates: any) {
    const settlementCurrencyId = this.settlementCurrencies.find(
      (p) => p.CurrencyShortName == agencyRates.SettlementCurrency
    )?.Currency_Id;
    const currencyId = this.currencies.find(
      (p) => p.CurrencyShortName == agencyRates.CurrencyShortName
    )?.Id;
    this.manageAgencyRatesForm.patchValue({
      SettlementCurrency: settlementCurrencyId,
      Currency_ID: currencyId,
      Bid_Rate: agencyRates?.BidRateUSD,
      Offer_Rate: agencyRates?.OfferRateUSD,
      Revalution_Rate: agencyRates?.RevaluationRateUSD,
      Settelment_Rate: agencyRates?.SettelmentRateUSD,
      AutoRateflag: agencyRates?.AutoRateFlag,
      ExecutionTime: agencyRates?.Exec_Duration,
    });
    this.openAgencyRatesModal('edit');
  }

  isCheckOfferRate(offerRate: number): boolean {
    const bidRate = Number(this.f.Bid_Rate);
    return Number(offerRate) <= bidRate;
  }

  isCheckSettlementRate(settlementRate: number): boolean {
    const bidRate = Number(this.f.Bid_Rate);
    return Number(settlementRate) <= bidRate;
  }

  isCheckExecutionDuration(duration: number): boolean {
    return duration <= 14;
  }

  updateAgencyRates(): void {
    const updatedAgencyRates = this.agencyRates
      .filter((con) => con.isChecked)
      .map((con) => {
        const settlementCurrencyId = this.settlementCurrencies.find(
          (c) => c.CurrencyShortName === con.SettlementCurrency
        )?.Currency_Id;
        const currencyId = this.currencies.find(
          (c) => c.CurrencyShortName === con.CurrencyShortName
        )?.Id;

        return {
          SettlementCurrency: settlementCurrencyId,
          Currency_ID: currencyId,
          Bid_Rate: Number(con?.BidRateUSD),
          Offer_Rate: Number(con?.OfferRateUSD),
          Revalution_Rate: Number(con?.RevaluationRateUSD),
          Settelment_Rate: Number(con?.SettelmentRateUSD),
          ExecutionTime: Number(con?.Exec_Duration),
          AutoRateflag: con?.AutoRateFlag,
          EnteredBy: '1001000',
        };
      });

    if (updatedAgencyRates?.length) {
      this.agencyService
        .updateAgencyRates(
          getCacheData('admin_token'),
          getCacheData('username'),
          updatedAgencyRates
        )
        .subscribe({
          next: (res: any) => {
            this.modalService.hide();
            alert('Currency Rate Update Successfully.');
            this.getAllCurrencies();
          },
          error: (err) => {
            this.modalService.hide();
            alert(err?.error?.message || 'Something went wrong');
          },
          complete: () => {
            this.modalService.hide();
          },
        });
    }
  }

  resetAgencyRates() {
    this.manageAgencyRatesForm.reset({
      SettlementCurrency: 'None',
      Currency_ID: 'None',
    });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
