import { Component, TemplateRef, ViewChild } from '@angular/core';
import { CancelPaidTransactionService } from '../services/cancel-paid-transaction.service';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';

@Component({
  selector: 'app-cancel-paid-transaction',
  templateUrl: './cancel-paid-transaction.component.html',
  styleUrls: ['./cancel-paid-transaction.component.css'],
})
export class CancelPaidTransactionComponent {
  @ViewChild('viewTransactionLog') viewTransactionLogModal;
  bsModalRef: BsModalRef;

  agentAccountList: any = [];
  selectedAgent: any = 0;
  agentBranchList: any = [];
  selectedAgentBranch: any;
  cancelledTypeList: any = [
    {
      id: 0,
      type: 'All',
    },
    {
      id: 10,
      type: 'CancelWithOutCharges',
    },
    {
      id: 11,
      type: 'CancelWithCharges',
    },
  ];
  selectedCancelledType: any = 0;
  statusList: any = [
    {
      id: -1,
      type: 'All',
    },
    {
      id: 0,
      type: 'Pending',
    },
    {
      id: 1,
      type: 'Processed',
    },
  ];
  selectedStatus: any = -1;
  fromDate: any = '';
  toDate: any = '';
  transactionNo: any = '';
  allCancelPaidTransactions: any = [];
  transactionsLogList: any = [];
  tableCancelledType: any = [
    {
      id: 10,
      type: 'Cancellation / Refund charges',
    },
    {
      id: 11,
      type: 'Cancellation / do not refund charges',
    },
  ];
  selectedCancelledTypeList: any = 10;

  constructor(
    private cancelPaidSvc: CancelPaidTransactionService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getAgentAcountList();
    this.getAgentBranchList(0);
  }

  getAgentAcountList() {
    this.cancelPaidSvc.getAgentAcountList().subscribe({
      next: (res: any) => {
        this.agentAccountList = res;
      },
      error: (err) => {
        this.agentAccountList = [];
      },
      complete: () => {},
    });
  }

  onAgentAccountChange(agentId: any) {
    if (agentId != 0) {
      this.getAgentBranchList(agentId);
    }
  }

  getAgentBranchList(agentId: any = 0) {
    this.cancelPaidSvc.getAgentBranchList(agentId).subscribe({
      next: (res: any) => {
        this.agentBranchList = res;
        this.selectedAgentBranch = res?.[0]?.ID;
      },
      error: (err) => {
        this.agentBranchList = [];
      },
      complete: () => {},
    });
  }

  displayByDate() {
    this.cancelPaidSvc
      .getAllRecordByStartAndEndDates(
        this.selectedAgent,
        this.selectedAgentBranch,
        this.selectedCancelledType,
        this.selectedStatus,
        this.fromDate,
        this.toDate
      )
      .subscribe({
        next: (res: any) => {
          this.allCancelPaidTransactions = res;
        },
        error: () => {
          this.allCancelPaidTransactions = [];
        },
        complete: () => {},
      });
  }

  filterByTransactioNo() {
    if (!this.transactionNo) {
      alert('Please Fill Transaction Number!');
      return;
    }

    this.cancelPaidSvc
      .getRecordByTransactionNumber(0, this.transactionNo)
      .subscribe({
        next: (res: any) => {
          this.allCancelPaidTransactions = res;
        },
        error: () => {
          this.allCancelPaidTransactions = [];
        },
        complete: () => {},
      });
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openTransactionLogModal(transactionNo: any) {
    if (transactionNo) {
      this.getTransactionLogByTransactionNo(transactionNo);
    }
  }

  getTransactionLogByTransactionNo(transactionNumber: any = 0) {
    this.cancelPaidSvc
      .getTransactionLogByTransactionNo(transactionNumber)
      .subscribe({
        next: (res: any) => {
          this.transactionsLogList = res;
          this.openModal(this.viewTransactionLogModal);
        },
        error: () => {
          this.transactionsLogList = [];
        },
        complete: () => {},
      });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
