import { Component, TemplateRef, ViewChild } from '@angular/core';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { getCacheData } from '../utils/helper';
import { CountryWiseCommissionService } from '../services/country-wise-commission.service';

@Component({
  selector: 'app-country-wise-commission',
  templateUrl: './country-wise-commission.component.html',
  styleUrls: ['./country-wise-commission.component.css'],
})
export class CountryWiseCommissionComponent {
  @ViewChild('manageCountryWise') manageCountryWiseModal;
  bsModalRef: BsModalRef;

  type: any = 'add';

  sendingCountries: any = [];
  selectedSendCountry: any = 'All';

  receivingCountries: any = [];
  selectedReceivingCountry: any = 'All';

  modeofPayments: any = [];
  selectedModeofPayment: any = 'All';

  countryWiseCommissions: any = [];

  sendInsertCountries: any = [];
  sendInsertCurrencies: any = [];
  receiveInsertCountries: any = [];
  receiveInsertCurrencies: any = [];
  totalCommissionTypes: any = [];
  sendCommissionTypes: any = [];
  receiveCommissionTypes: any = [];
  sendCurrencyTypes: any = [];
  receiveCurrencyTypes: any = [];

  manageCountryWiseCommissionForm = new FormGroup({
    PayInCommissionType: new FormControl('', Validators.required),
    PayInCurrencyType: new FormControl('', Validators.required),
    PayInCommissionAmount: new FormControl('', Validators.required),
    PayOutCommissionType: new FormControl('', Validators.required),
    PayOutCurrencyType: new FormControl('', Validators.required),
    PayOutCommissionAmount: new FormControl('', Validators.required),
    PayCommissionType: new FormControl('', Validators.required),
    BaseToBaseCommissionAmount: new FormControl('', Validators.required),
    EndRange: new FormControl('', Validators.required),
    StartRange: new FormControl('', Validators.required),
    ModeOfPayment_ID: new FormControl('None', Validators.required),
    PayOutCurrencyID: new FormControl('None', Validators.required),
    PayInCurrencyID: new FormControl('None', Validators.required),
    PayOutCountryID: new FormControl('None', Validators.required),
    PayInCountryID: new FormControl('None', Validators.required),
    ID: new FormControl(),
  });

  constructor(
    private countryCommissionService: CountryWiseCommissionService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getSendingCountry();
    this.getInsertSendCountry();
  }

  getSendingCountry() {
    this.countryCommissionService
      .getSendingCountry(getCacheData('admin_token'), getCacheData('username'))
      .subscribe({
        next: (res: any) => {
          this.sendingCountries = res;
        },
        error: () => {
          this.sendingCountries = [];
        },
        complete: () => {},
      });
  }

  getReceivingCountry(countryId: any = 0) {
    this.countryCommissionService
      .getReceivingCountry(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.receivingCountries = res;
        },
        error: () => {
          this.receivingCountries = [];
        },
        complete: () => {},
      });
  }

  getModeofPayment(countryId: any = 0) {
    this.countryCommissionService
      .getModeofPayment(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.modeofPayments = res;
        },
        error: () => {
          this.modeofPayments = [];
        },
        complete: () => {},
      });
  }

  getAllCountryWiseCommission(
    payInCountryId: any = 0,
    payOutCountryId: any = 0,
    modeofPaymentId: any = 0
  ) {
    this.countryCommissionService
      .getAllCountryWiseCommission(
        getCacheData('admin_token'),
        getCacheData('username'),
        payInCountryId,
        payOutCountryId,
        modeofPaymentId
      )
      .subscribe({
        next: (res: any) => {
          this.countryWiseCommissions = res;
          this.formatCountryWiseCommission(this.countryWiseCommissions);
        },
        error: () => {
          this.countryWiseCommissions = [];
        },
        complete: () => {},
      });
  }

  formatCountryWiseCommission(array: any = []) {
    for (let index = 0; index < array.length; index++) {
      const element = array[index];
      if (element?.StartRange !== null && element?.StartRange !== undefined) {
        element.StartRange = this.formatWithThreeDecimalPlaces(
          element.StartRange
        );
      }
      if (element?.EndRange !== null && element?.EndRange !== undefined) {
        element.EndRange = this.formatWithThreeDecimalPlaces(element.EndRange);
      }
      if (
        element?.BaseToBaseCommissionAmount !== null &&
        element?.BaseToBaseCommissionAmount !== undefined
      ) {
        element.BaseToBaseCommissionAmount = this.formatWithThreeDecimalPlaces(
          element.BaseToBaseCommissionAmount
        );
      }
      if (
        element?.BaseToUSDCommissionAmount !== null &&
        element?.BaseToUSDCommissionAmount !== undefined
      ) {
        element.BaseToUSDCommissionAmount = this.formatWithThreeDecimalPlaces(
          element.BaseToUSDCommissionAmount
        );
      }
      if (
        element?.SendCommissionAmount !== null &&
        element?.SendCommissionAmount !== undefined
      ) {
        element.SendCommissionAmount = this.formatWithThreeDecimalPlaces(
          element.SendCommissionAmount
        );
      }

      if (
        element?.ReceiveCommissionAmount !== null &&
        element?.ReceiveCommissionAmount !== undefined
      ) {
        element.ReceiveCommissionAmount = this.formatWithThreeDecimalPlaces(
          element.ReceiveCommissionAmount
        );
      }

      if (
        element?.PayOutFixAmount !== null &&
        element?.PayOutFixAmount !== undefined
      ) {
        element.PayOutFixAmount = this.formatWithThreeDecimalPlaces(
          element.PayOutFixAmount
        );
      }

      if (
        element?.PayOutMinAmount !== null &&
        element?.PayOutMinAmount !== undefined
      ) {
        element.PayOutMinAmount = this.formatWithThreeDecimalPlaces(
          element.PayOutMinAmount
        );
      }

      if (
        element?.PayOutMaxAmount !== null &&
        element?.PayOutMaxAmount !== undefined
      ) {
        element.PayOutMaxAmount = this.formatWithThreeDecimalPlaces(
          element.PayOutMaxAmount
        );
      }
    }
  }

  formatWithThreeDecimalPlaces(value: number | string): string {
    if (value === null || value === undefined) {
      return '';
    }

    const stringValue = value.toString();
    const [integerPart, fractionalPart] = stringValue.split('.');

    if (fractionalPart) {
      // Pad with zeros to ensure 3 decimal places
      return `${integerPart}.${fractionalPart.padEnd(3, '0').slice(0, 3)}`;
    } else {
      // If no decimal places, add 3 zeros
      return `${integerPart}.000`;
    }
  }

  get f() {
    return this.manageCountryWiseCommissionForm.value;
  }

  onSendCountryChange() {
    if (this.selectedSendCountry != 'All') {
      this.getReceivingCountry(this.selectedSendCountry);
    }
  }

  onReceiveCountryChange() {
    if (this.selectedReceivingCountry != 'All') {
      this.getModeofPayment(this.selectedReceivingCountry);
    }
  }

  searchCommission() {
    let sendCountryId: number | null = 0;
    let receiveCountryId: number | null = 0;
    let modeofPaymentId: number | null = 0;

    if (this.selectedSendCountry !== 'All') {
      sendCountryId = this.selectedSendCountry;
    }

    if (this.selectedReceivingCountry !== 'All') {
      receiveCountryId = this.selectedReceivingCountry;
    }

    if (this.selectedModeofPayment !== 'All') {
      modeofPaymentId = this.selectedModeofPayment;
    }

    this.getAllCountryWiseCommission(
      sendCountryId,
      receiveCountryId,
      modeofPaymentId
    );
  }

  getInsertSendCountry() {
    this.countryCommissionService
      .getInsertSendCountry(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.sendInsertCountries = res;
          this.getInsertReceiveCountry();
          this.getInsertSendCurrency();
          this.getInsertReceiveCurrency();
          this.getTotalCommissionType();
        },
        error: () => {
          this.sendInsertCountries = [];
        },
        complete: () => {},
      });
  }

  getInsertReceiveCountry() {
    this.countryCommissionService
      .getInsertReceiveCountry(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.receiveInsertCountries = res;
          this.getReceivingCountry(res?.[0]?.CountryId);
          this.getModeofPayment(res?.[0]?.CountryId);
        },
        error: () => {
          this.receiveInsertCountries = [];
        },
        complete: () => {},
      });
  }

  getInsertSendCurrency(payInCountryId = 0) {
    this.countryCommissionService
      .getInsertSendCurrency(
        getCacheData('admin_token'),
        getCacheData('username'),
        payInCountryId
      )
      .subscribe({
        next: (res: any) => {
          this.sendInsertCurrencies = res;
          this.getSendCurrencyType();
        },
        error: () => {
          this.sendInsertCurrencies = [];
        },
        complete: () => {},
      });
  }

  getInsertReceiveCurrency(payInCountryId = 0) {
    this.countryCommissionService
      .getInsertSendCurrency(
        getCacheData('admin_token'),
        getCacheData('username'),
        payInCountryId
      )
      .subscribe({
        next: (res: any) => {
          this.receiveInsertCurrencies = res;
        },
        error: () => {
          this.receiveInsertCurrencies = [];
        },
        complete: () => {},
      });
  }

  getTotalCommissionType() {
    this.countryCommissionService
      .getTotalCommissionType(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.totalCommissionTypes = res;
          this.getSendCommissionType();
          this.getReceiveCommissionType();
        },
        error: () => {
          this.totalCommissionTypes = [];
        },
        complete: () => {},
      });
  }

  getSendCommissionType() {
    if (this.totalCommissionTypes?.length) {
      this.sendCommissionTypes = this.totalCommissionTypes;
    }
  }

  getReceiveCommissionType() {
    if (this.totalCommissionTypes?.length) {
      this.receiveCommissionTypes = this.totalCommissionTypes;
    }
  }

  getSendCurrencyType() {
    this.countryCommissionService
      .getCurrencyType(getCacheData('admin_token'), getCacheData('username'))
      .subscribe({
        next: (res: any) => {
          this.sendCurrencyTypes = res;
          this.getReceiveCurrencyType();
        },
        error: () => {
          this.sendCurrencyTypes = [];
        },
        complete: () => {},
      });
  }

  getReceiveCurrencyType() {
    if (this.sendCurrencyTypes?.length) {
      this.receiveCurrencyTypes = this.sendCurrencyTypes;
    }
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openCountryWiseCommissionModal(type: any) {
    this.type = type;

    if (type === 'add') {
      this.manageCountryWiseCommissionForm.reset({
        PayInCountryID: 'None',
        PayOutCountryID: 'None',
        PayInCurrencyID: 'None',
        PayOutCurrencyID: 'None',
        PayCommissionType: 'None',
        ModeOfPayment_ID: 'None',
        PayInCommissionType: 'None',
        PayOutCommissionType: 'None',
        PayInCurrencyType: 'None',
        PayOutCurrencyType: 'None',
      });
    }

    this.openModal(this.manageCountryWiseModal);
  }

  onSendInsertCountryChange(countryId: any) {
    if (countryId != 'All') {
      this.getInsertReceiveCountry();
      this.getInsertSendCurrency(countryId);
      this.getTotalCommissionType();
    }
  }

  onReceiveInsertCountryChange(countryId: any) {
    if (countryId != 'None') {
      this.getInsertReceiveCurrency(countryId);
      this.getModeofPayment(countryId);
    }
  }

  onSendCurrencyChange(countryId: any) {
    if (countryId != 'None') {
      this.getSendCurrencyType();
    }
  }

  onReceiveCurrencyChange(countryId: any) {
    if (countryId != 'None') {
      this.getReceiveCurrencyType();
    }
  }

  addCountryWiseCommission() {
    if (!this.manageCountryWiseCommissionForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    if (this.f.PayInCountryID == 'None') {
      alert('Please Select Send Country!');
      return;
    }

    if (this.f.PayInCurrencyID == 'None') {
      alert('Please Select Send Currency!');
      return;
    }

    if (this.f.PayOutCountryID == 'None') {
      alert('Please Select Receive Country!');
      return;
    }
    if (this.f.PayOutCurrencyID == 'None') {
      alert('Please Select Receive Currency!');
      return;
    }
    if (this.f.ModeOfPayment_ID == 'None') {
      alert('Please Select Mode of Payment!');
      return;
    }

    if (this.f.EndRange < this.f.StartRange) {
      alert('End Range should be greater than Start Range!');
      return;
    }
    if (this.f.PayCommissionType == 'None') {
      alert('Please Select Total Commission Type!');
      return;
    }
    if (this.f.PayInCommissionType == 'None') {
      alert('Please Select Send Commission Type!');
      return;
    }
    if (this.f.PayInCurrencyType == 'None') {
      alert('Please Select Send Currency Type!');
      return;
    }
    if (this.f.PayOutCommissionType == 'None') {
      alert('Please Select Receive Commission Type!');
      return;
    }
    if (this.f.PayOutCurrencyType == 'None') {
      alert('Please Select Receive Currency Type!');
      return;
    }

    let payload;
    payload = {
      PayInCountryID: this.f.PayInCountryID,
      PayInCurrencyID: this.f.PayInCurrencyID,
      PayOutCountryID: this.f.PayOutCountryID,
      PayOutCurrencyID: this.f.PayOutCurrencyID,
      ModeOfPayment_ID: this.f.ModeOfPayment_ID,
      StartRange: this.f.StartRange,
      EndRange: this.f.EndRange,
      BaseToBaseCommissionAmount: this.f.BaseToBaseCommissionAmount,
      PayCommissionType: this.f.PayCommissionType,
      PayInCommissionType: this.f.PayInCommissionType,
      PayInCurrencyType: this.f.PayInCurrencyType,
      PayInCommissionAmount: this.f.PayInCommissionAmount,
      PayOutCommissionType: this.f.PayOutCommissionType,
      PayOutCurrencyType: this.f.PayOutCurrencyType,
      PayOutCommissionAmount: this.f.PayOutCommissionAmount,
      EnteredBy: 'none',
    };

    if (this.type === 'edit') {
      payload = {
        ...payload,
        ID: this.f.ID,
      };
    }

    const serviceMethod =
      this.type === 'add'
        ? 'addCountryWiseCommission'
        : 'updateCountryWiseCommission';

    this.countryCommissionService[serviceMethod](
      getCacheData('admin_token'),
      getCacheData('username'),
      payload
    ).subscribe({
      next: (res: any) => {
        this.modalService.hide();
        this.getAllCountryWiseCommission();
        this.getSendingCountry();
      },
      error: (err) => {
        this.modalService.hide();
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  async editCountryWiseCommission(country: any) {
    console.log('editCountryWiseCommissioncountry: ', country);
    const payInCountryID = this.sendInsertCountries.find(
      (c) => c.CountryName == country.SendCountry
    )?.CountryId;

    const payInCurrencyId = this.sendInsertCurrencies.find(
      (c) => c.CurrencyShortName == country.SendCurrency
    )?.Currency_Id;

    const payOutCountryID = this.receiveInsertCountries.find(
      (c) => c.CountryName == country.ReceiveCountry
    )?.CountryId;

    const payOutCurrencyId = this.receiveInsertCurrencies.find(
      (c) => c.CurrencyShortName == country.ReceiveCurrency
    )?.Currency_Id;

    const modeofPaymentId = this.modeofPayments.find(
      (c) => c.ModeOfPayment_Name == country.ModeOfPayment
    )?.ModeOfPayment_ID;

    const payCommissionType = this.totalCommissionTypes.find(
      (c) => c.CommissionType == country.CommissionType
    )?.ID;

    const sendCommissionType = this.sendCommissionTypes.find(
      (c) => c.CommissionType == country.SendCommissionType
    )?.ID;

    const payInCurrencyType = this.sendCurrencyTypes.find(
      (c) => c.CurrencyType == country.SendCurrencyType
    )?.ID;

    const payOutCommissionType = this.receiveCommissionTypes.find(
      (c) => c.CommissionType == country.ReceiveCommissionType
    )?.ID;

    const payOutCurrencyType = this.receiveCurrencyTypes.find(
      (c) => c.CurrencyType == country.ReceiveCurrencyType
    )?.ID;

    this.manageCountryWiseCommissionForm.patchValue({
      ID: country?.ID,
      PayInCountryID: payInCountryID,
      PayInCurrencyID: payInCurrencyId,
      PayOutCountryID: payOutCountryID,
      PayOutCurrencyID: payOutCurrencyId,
      ModeOfPayment_ID: modeofPaymentId,
      StartRange: country?.StartRange,
      EndRange: country?.EndRange,
      BaseToBaseCommissionAmount: country?.BaseToBaseCommissionAmount,
      PayCommissionType: payCommissionType,
      PayInCommissionType: sendCommissionType,
      PayInCurrencyType: payInCurrencyType,
      PayInCommissionAmount: country?.SendCommissionAmount,
      PayOutCommissionType: payOutCommissionType,
      PayOutCurrencyType: payOutCurrencyType,
      PayOutCommissionAmount: country?.ReceiveCommissionAmount,
    });
    this.openCountryWiseCommissionModal('edit');
  }

  update() {
    const deletedCountries = this.getDeletedCountries();
    const updatedCountries = this.getUpdatedCountries();

    this.deleteCountryWiseCommission(deletedCountries);
    this.updatedCountryWiseCommission(updatedCountries);
  }

  getDeletedCountries() {
    return this.countryWiseCommissions
      .filter((con) => con.isDeleted)
      .map((con) => ({ ID: con?.ID }));
  }

  getUpdatedCountries() {
    const updatedCountries = this.countryWiseCommissions
      .filter((con) => con.isChecked)
      .map((con) => ({
        ID: con?.ID,
        StartRange: Number(con?.StartRange),
        EndRange: Number(con?.EndRange),
        BaseToBaseCommissionAmount: con?.BaseToBaseCommissionAmount,
        BaseToUSDCommissionAmount: con?.BaseToUSDCommissionAmount,
        SendCommissionAmount: con?.SendCommissionAmount,
        ReceiveCommissionAmount: con?.ReceiveCommissionAmount,
        UserID: 'string',
        UpdatedStartRange: Number(con?.StartRange),
        UpdatedEndRange: Number(con?.EndRange),
        UpdatedBaseToBaseCommissionAmount: Number(
          con?.BaseToBaseCommissionAmount
        ),
        UpdatedBaseToUSDCommissionAmount: Number(
          con?.BaseToUSDCommissionAmount
        ),
        UpdatedSendCommissionAmount: Number(con?.SendCommissionAmount),
        UpdatedReceiveCommissionAmount: Number(con?.ReceiveCommissionAmount),
      }));

    return {
      IsValidUser: this.validUser,
      IsValidAuthorizeUser: this.validAuthorizeUser,
      CountryGridUpdateRequestList: updatedCountries,
    };
  }

  deleteCountryWiseCommission(deletedCountries) {
    if (deletedCountries?.length) {
      this.countryCommissionService
        .deleteCountryWiseCommission(
          getCacheData('admin_token'),
          getCacheData('username'),
          deletedCountries
        )
        .subscribe({
          next: (res: any) => {
            this.searchCommission();
            alert('Country Wise Commission Deleted Successfully.');
          },
          error: (err) => {
            alert(err?.error?.message || 'Something went wrong');
          },
          complete: () => {},
        });
    }
  }

  updatedCountryWiseCommission(updatedCountries) {
    if (updatedCountries?.CountryGridUpdateRequestList?.length) {
      this.countryCommissionService
        .updateCountryWiseGrid(
          getCacheData('admin_token'),
          getCacheData('username'),
          updatedCountries
        )
        .subscribe({
          next: (res: any) => {
            this.searchCommission();
            alert('Country Wise Commission Update Successfully.');
          },
          error: (err) => {
            alert(err?.error?.message || 'Something went wrong');
          },
          complete: () => {},
        });
    }
  }

  get validUser(): boolean {
    const subUserGroupId = getCacheData('SubUserGroup_Id');
    const gUserGroup = getCacheData('G_UserGroup');

    if (subUserGroupId && gUserGroup) {
      const subUserGroupIdNumber = parseInt(subUserGroupId, 10);
      const gUserGroupNumber = parseInt(gUserGroup, 10);

      return subUserGroupIdNumber === 6 && gUserGroupNumber === 1;
    }

    return false;
  }

  get validAuthorizeUser(): boolean {
    const subUserGroupId = getCacheData('SubUserGroup_Id');
    const gUserGroup = getCacheData('G_UserGroup');

    if (subUserGroupId && gUserGroup) {
      const subUserGroupIdNumber = parseInt(subUserGroupId, 10);
      const gUserGroupNumber = parseInt(gUserGroup, 10);

      return subUserGroupIdNumber === 1 && gUserGroupNumber === 1;
    }

    return false;
  }

  isValidEndRange(start_range: number, end_range: number): boolean {
    return Number(start_range) < Number(end_range);
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
