import { Component, ElementRef, ViewChild } from '@angular/core';
import { CtaHoldSummaryService } from '../services/cta-hold-summary.service';
import * as XLSX from 'xlsx';

@Component({
  selector: 'app-cta-hold-summary',
  templateUrl: './cta-hold-summary.component.html',
  styleUrls: ['./cta-hold-summary.component.css'],
})
export class CtaHoldSummaryComponent {
  @ViewChild('ctaHoldSummaryTable') ctaHoldSummaryTable: ElementRef;

  countryList: any = [];
  selectedCountry: any;
  agentList: any = [];
  selectedAgent: any;
  toDate: any = '';
  allCtaHoldSummary: any = [];

  constructor(private ctaHoldSummarySvc: CtaHoldSummaryService) {}

  ngOnInit(): void {
    this.getCountryList();
    this.getAgentList(0);
  }

  getCountryList() {
    this.ctaHoldSummarySvc.getCountryList().subscribe({
      next: (res: any) => {
        this.countryList = res;
        this.selectedCountry = res?.[0]?.CountryId;
      },
      error: (err) => {
        this.countryList = [];
      },
      complete: () => {},
    });
  }

  onCountryChange(countryId: any) {
    if (countryId != 0) {
      this.getAgentList(countryId);
    }
  }

  getAgentList(countryId: any = 0) {
    this.ctaHoldSummarySvc.getAgentList(countryId).subscribe({
      next: (res: any) => {
        this.agentList = res;
        this.selectedAgent = res?.[0]?.Agent_ID;
      },
      error: (err) => {
        this.agentList = [];
      },
      complete: () => {},
    });
  }

  displayByDate() {
    if (!this.toDate) {
      alert('Please Select Date!');
      return;
    }
    this.ctaHoldSummarySvc
      .getAllRecord(this.toDate, this.selectedCountry, this.selectedAgent)
      .subscribe({
        next: (res: any) => {
          this.allCtaHoldSummary = res;
          if (this.allCtaHoldSummary?.length) {
            this.allCtaHoldSummary.forEach((item) => {
              if (typeof item === 'object' && item !== null) {
                item.countryName = this.sanitizeHTML(item.countryName);
                item.agentname = this.sanitizeHTML(item.agentname);
                item.totaltrxn = this.sanitizeHTML(item.totaltrxn);
                item.LCamount = this.sanitizeHTML(item.LCamount);
                item.USDamount = this.sanitizeHTML(item.USDamount);
                item.isBold = false;
                item.isBold =
                  item.agentname.toLowerCase().includes('country total') ||
                  item.agentname.toLowerCase().includes('grand total');
              }
            });
          }
          console.log('allCtaHoldSummary: ', this.allCtaHoldSummary);
        },
        error: () => {
          this.allCtaHoldSummary = [];
        },
        complete: () => {},
      });
  }

  sanitizeHTML(htmlString: string): string {
    const doc = new DOMParser().parseFromString(htmlString, 'text/html');
    return doc.body.textContent || '';
  }

  exportToExcel() {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.ctaHoldSummaryTable.nativeElement
    );

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `RA_CTA_Hold_Summary${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
