import { Component, OnInit, TemplateRef, ViewChild } from '@angular/core';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { ManageCountryService } from '../manage-country.service';
import { getCacheData } from '../utils/helper';
import { ManageCityService } from '../services/manage-city.service';

@Component({
  selector: 'app-manage-city',
  templateUrl: './manage-city.component.html',
  styleUrls: ['./manage-city.component.css'],
})
export class ManageCityComponent implements OnInit {
  @ViewChild('manageCity') manageCityModal;
  bsModalRef: BsModalRef;

  countries: any = [];
  filterCounteries: any = [];
  selectedCountry: any = 'All';

  type: any = 'add';

  cities: any = [];

  manageCityForm = new FormGroup({
    CityID: new FormControl(),
    CountryID: new FormControl('None', Validators.required),
    CityName: new FormControl('', Validators.required),
    EnteredBy: new FormControl('none'),
    Message: new FormControl('none'),
  });

  constructor(
    private countryService: ManageCountryService,
    private cityService: ManageCityService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getCountries();
    this.getAllCities();
  }

  getAllCities(countryId = 0) {
    this.cityService
      .getAllCities(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.cities = res;
        },
        error: (err) => {
          this.cities = [];
        },
        complete: () => {},
      });
  }

  getCountries() {
    this.countryService
      .getCountries(getCacheData('admin_token'), getCacheData('username'))
      .subscribe({
        next: (res: any) => {
          this.countries = res;
          this.filterCounteries = [...res];
        },
        error: (err) => {
          this.countries = [];
        },
        complete: () => {},
      });
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openCityModal(type: any) {
    if (type == 'add') {
      this.manageCityForm.reset({
        CountryID: 'None',
        CityName: '',
      });
    }
    this.type = type;
    this.openModal(this.manageCityModal);
  }

  searchCity() {
    let countryId: number | null = 0;

    if (this.selectedCountry !== 'All') {
      countryId = this.selectedCountry;
    }

    this.getAllCities(countryId);
  }

  get f() {
    return this.manageCityForm.value;
  }

  addCity() {
    if (!this.manageCityForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    if (this.f.CountryID == 'None') {
      alert('Please Select Country!');
      return;
    }

    let payload;
    payload = {
      CountryID: this.f.CountryID,
      CityName: this.f.CityName,
      DtEntered: new Date(),
      TmEntered: new Date(),
      EnteredBy: 'none',
      Message: 'none',
    };

    if (this.type === 'edit') {
      payload = {
        ...payload,
        CityID: this.f.CityID,
      };
    }

    const serviceMethod = this.type === 'add' ? 'addCity' : 'updateCity';

    this.cityService[serviceMethod](
      getCacheData('admin_token'),
      getCacheData('username'),
      payload
    ).subscribe({
      next: (res: any) => {
        this.modalService.hide();
        this.getAllCities();
      },
      error: (err) => {
        this.modalService.hide();
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  editCity(city: any) {
    const countryId = this.countries.find(
      (c) => c.CountryName == city.CountryName
    )?.CountryId;
    this.manageCityForm.patchValue({
      CityID: city?.ID,
      CountryID: countryId,
      CityName: city?.CityName,
    });
    this.openCityModal('edit');
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
