import { Component, TemplateRef, ViewChild } from '@angular/core';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { ManageCountryService } from '../manage-country.service';
import { getCacheData } from '../utils/helper';
import { ManageLocationService } from '../services/manage-location.service';
import { ManageCityService } from '../services/manage-city.service';

@Component({
  selector: 'app-manage-location',
  templateUrl: './manage-location.component.html',
  styleUrls: ['./manage-location.component.css'],
})
export class ManageLocationComponent {
  @ViewChild('manageLocation') manageLocationModal;
  bsModalRef: BsModalRef;

  countries: any = [];
  filterCounteries: any = [];
  selectedCountry: any = 'All';

  type: any = 'add';

  locations: any = [];

  cities: any = [];
  filterCities: any = [];
  selectedCity: any = 'All';

  manageLocationForm = new FormGroup({
    LocationID: new FormControl(),
    CityID: new FormControl('All', Validators.required),
    CountryID: new FormControl('None', Validators.required),
    LocationName: new FormControl('', Validators.required),
  });

  constructor(
    private countryService: ManageCountryService,
    private cityService: ManageCityService,
    private locationService: ManageLocationService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getCountries();
    this.getAllLocations();
    // this.getAllCities();
  }

  getAllLocations(countryId = 0, cityId = 0) {
    this.locationService
      .getAllLocations(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId,
        cityId
      )
      .subscribe({
        next: (res: any) => {
          this.locations = res;
        },
        error: (err) => {
          this.locations = [];
        },
        complete: () => {},
      });
  }

  getAllCities(countryId: any = 0, location: any = ''): void {
    this.cityService
      .getAllCities(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.cities = res;
          if (this.type === 'add' && !location) {
            this.handleAddCase(res);
          } else {
            this.handleEditCase(location, countryId);
          }
        },
        error: (err) => {
          this.cities = [];
        },
        complete: () => {},
      });
  }

  private handleAddCase(res: any[]): void {
    this.filterCities = [...res];
    this.selectedCity = res?.[0]?.ID;
  }

  private handleEditCase(location: any, countryId: any): void {
    const cityId = this.getCityIdFromName(location.CityName);
    this.setFormValues(location, countryId, cityId);
    this.openLocationModal('edit');
  }

  private setFormValues(location: any, countryId: any, cityId: any): void {
    this.manageLocationForm.patchValue({
      LocationName: location?.LocationName,
      LocationID: location?.ID,
      CountryID: countryId,
      CityID: cityId,
    });
  }

  private getCityIdFromName(cityName: string): number | undefined {
    return this.cities.find((ci) => ci.CityName === cityName)?.ID;
  }

  getCountries() {
    this.countryService
      .getCountries(getCacheData('admin_token'), getCacheData('username'))
      .subscribe({
        next: (res: any) => {
          this.countries = res;
          this.selectedCountry = res?.[0]?.CountryId;
          if (this.type == 'add') {
            this.getAllCities(this.selectedCountry);
          }
          this.filterCounteries = [...res];
        },
        error: (err) => {
          this.countries = [];
        },
        complete: () => {},
      });
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openLocationModal(type: any) {
    this.type = type;

    if (type === 'add') {
      this.manageLocationForm.reset({
        CountryID: 'None',
        LocationName: '',
        CityID: '',
      });
      if (this.f.CountryID === 'None') {
        this.cities = [];
      }
    }

    this.openModal(this.manageLocationModal);
  }

  searchLocation() {
    let countryId: number | null = 0;
    let cityId: number | null = 0;

    if (this.selectedCountry !== 'All') {
      countryId = this.selectedCountry;
    }
    if (this.selectedCity !== 'All') {
      cityId = this.selectedCity;
    }

    this.getAllLocations(countryId, cityId);
  }

  get f() {
    return this.manageLocationForm.value;
  }

  addLocation() {
    if (!this.manageLocationForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    if (this.f.CountryID == 'All') {
      alert('Please Select Country!');
      return;
    }

    if (this.f.CityID == 'All') {
      alert('Please Select City!');
      return;
    }

    let payload;
    payload = {
      CityID: this.f.CityID,
      LocationName: this.f.LocationName,
      CountryID: this.f.CountryID,
      DtEntered: new Date(),
      TmEntered: new Date(),
      EnteredBy: 'none',
      Message: 'none',
    };

    if (this.type === 'edit') {
      payload = {
        ...payload,
        LocationID: this.f.LocationID,
      };
    }

    const serviceMethod =
      this.type === 'add' ? 'addLocation' : 'updateLocation';

    this.locationService[serviceMethod](
      getCacheData('admin_token'),
      getCacheData('username'),
      payload
    ).subscribe({
      next: (res: any) => {
        this.modalService.hide();
        this.getAllLocations();
        // this.getAllCities();
      },
      error: (err) => {
        this.modalService.hide();
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  editLocation(location: any) {
    if (location) {
      const countryId = this.countries.find(
        (c) => c.CountryName == location.CountryName
      )?.CountryId;
      this.getAllCities(countryId, location);
    }
  }

  onCountryChange(countryId: any = 'All') {
    this.getAllCities(countryId);
  }

  onModalCountryChange(countryId: any) {
    this.onCountryChange(countryId);
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
