import { Component, OnInit, TemplateRef, ViewChild } from '@angular/core';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { getCacheData } from '../utils/helper';
import { MarginsSpecificSendingCountryService } from '../services/margins-specific-sending-country.service';

@Component({
  selector: 'app-margins-specific-sending-country',
  templateUrl: './margins-specific-sending-country.component.html',
  styleUrls: ['./margins-specific-sending-country.component.css'],
})
export class MarginsSpecificSendingCountryComponent implements OnInit {
  @ViewChild('manageSpecificSendingCountry') manageSpecificSendingCountryModal;
  bsModalRef: BsModalRef;

  sendingCountries: any = [];
  selectedCountry: any = 'All';

  type: any = 'add';

  sendCountries: any = [];
  sendCurrencies: any = [];

  specificSendingCountries: any = [];

  manageSpecificSendingCountryForm = new FormGroup({
    ID: new FormControl(),
    PayInCountryId: new FormControl('None', Validators.required),
    PayInCurrencyId: new FormControl('', Validators.required),
    MarginPercent: new FormControl('', Validators.required),
    Max_AgentMargin: new FormControl('', Validators.required),
    Min_AgentMargin: new FormControl('', Validators.required),
    Max_LimitForPremiumRate: new FormControl('', Validators.required),
  });

  constructor(
    private specificCountryService: MarginsSpecificSendingCountryService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getAllSendingCountries();
    this.getMarginSendingCountry();
    this.getMarginSendCountry();
  }

  getAllSendingCountries(countryId = 0) {
    this.specificCountryService
      .getAllSendingCountries(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.specificSendingCountries = res;
          for (
            let index = 0;
            index < this.specificSendingCountries.length;
            index++
          ) {
            const element = this.specificSendingCountries[index];
            if (
              element?.MarginPercent !== null &&
              element?.MarginPercent !== undefined
            ) {
              element.MarginPercent = this.formatWithFourDecimalPlaces(
                element.MarginPercent
              );
            }
          }
        },
        error: (err) => {
          this.specificSendingCountries = [];
        },
        complete: () => {},
      });
  }

  formatWithFourDecimalPlaces(value: number | string): string {
    if (value === null || value === undefined) {
      return '';
    }

    const stringValue = value.toString();
    const [integerPart, fractionalPart] = stringValue.split('.');

    if (fractionalPart) {
      // Pad with zeros to ensure 4 decimal places
      return `${integerPart}.${fractionalPart.padEnd(4, '0').slice(0, 4)}`;
    } else {
      // If no decimal places, add two zeros
      return `${integerPart}.00`;
    }
  }

  getMarginSendingCountry() {
    this.specificCountryService
      .getMarginSendingCountry(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.sendingCountries = res;
        },
        error: (err) => {
          this.sendingCountries = [];
        },
        complete: () => {},
      });
  }

  getMarginSendCountry() {
    this.specificCountryService
      .getMarginSendCountry(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.sendCountries = res;
          this.getMarginSendingCurrency(res?.[0]?.CountryID);
        },
        error: (err) => {
          this.sendCountries = [];
        },
        complete: () => {},
      });
  }

  getMarginSendingCurrency(countryId = 0) {
    this.specificCountryService
      .getMarginSendingCurrency(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.sendCurrencies = res;
        },
        error: (err) => {
          this.sendCurrencies = [];
        },
        complete: () => {},
      });
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openSpecificCountryModal(type: any) {
    this.type = type;

    if (type === 'add') {
      this.manageSpecificSendingCountryForm.reset({
        PayInCountryId: 'None',
        PayInCurrencyId: 'None',
        MarginPercent: '',
        Max_AgentMargin: '',
        Max_LimitForPremiumRate: '',
        Min_AgentMargin: '',
      });
      // if (this.f.PayInCountryId === 'None') {
      //   this.sendCurrencies = [];
      // }
    }

    this.openModal(this.manageSpecificSendingCountryModal);
  }

  searchSpecficCountry() {
    let countryId: number | null = 0;

    if (this.selectedCountry !== 'All') {
      countryId = this.selectedCountry;
    }

    this.getAllSendingCountries(countryId);
  }

  get f() {
    return this.manageSpecificSendingCountryForm.value;
  }

  addSpecificCountry() {
    if (!this.manageSpecificSendingCountryForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    if (this.f.PayInCountryId == 'None') {
      alert('Please Select Send Country!');
      return;
    }

    if (this.f.PayInCurrencyId == '') {
      alert('Please Select Send Currency!');
      return;
    }

    if (
      Number(this.f.MarginPercent) <= 0.1 ||
      Number(this.f.MarginPercent) >= 1.5
    ) {
      alert('Margin Percent should not be greater than 100%!');
      return;
    }

    if (this.f.Max_AgentMargin <= this.f.Min_AgentMargin) {
      alert(
        'Maximum Agent Margin Should be greater than Minimum Agent Margin!'
      );
      return;
    }

    if (this.f.MarginPercent < this.f.Max_LimitForPremiumRate) {
      alert('Maximum Premium Rate Limit Should be less than Margin Percent!');
      return;
    }

    let payload;
    payload = {
      PayInCountryId: this.f.PayInCountryId,
      PayInCurrencyId: this.f.PayInCurrencyId,
      MarginPercent: Number(this.f.MarginPercent),
      Max_AgentMargin: Number(this.f.Max_AgentMargin),
      Min_AgentMargin: Number(this.f.Min_AgentMargin),
      Max_LimitForPremiumRate: Number(this.f.Max_LimitForPremiumRate),
      Active: true,
      Message: 'none',
    };

    if (this.type === 'edit') {
      payload = {
        ...payload,
        ID: this.f.ID,
      };
    }

    const serviceMethod =
      this.type === 'add'
        ? 'addSpecificSendingCountry'
        : 'updateSpecificSendingCountry';

    this.specificCountryService[serviceMethod](
      getCacheData('admin_token'),
      getCacheData('username'),
      payload
    ).subscribe({
      next: (res: any) => {
        this.modalService.hide();
        this.getAllSendingCountries();
        this.getMarginSendingCountry();
      },
      error: (err) => {
        this.modalService.hide();
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  editSpecificCountry(country: any) {
    const sendCountryId = this.sendCountries.find(
      (c) => c.BaseCountryName == country.SendCountryName
    )?.BaseCountryID;

    const sendCurrencyId = this.sendCurrencies.find(
      (c) => c.CurrencyShortName == country.SendCurrencyName
    )?.Currency_Id;

    this.manageSpecificSendingCountryForm.patchValue({
      ID: country?.ID,
      PayInCountryId: sendCountryId,
      PayInCurrencyId: sendCurrencyId,
      MarginPercent: country?.MarginPercent,
      Max_AgentMargin: country?.MaximumAgentMargin,
      Max_LimitForPremiumRate: country?.MaximumPremiumRateLimit,
      Min_AgentMargin: country?.MinimumAgentMargin,
    });
    this.openSpecificCountryModal('edit');
  }

  onSendCountryChange(countryId) {
    if (countryId != 'None') {
      this.getMarginSendingCurrency(countryId);
    }
  }

  isValidMarginPercent(percent: number): boolean {
    return percent >= 0.1 && percent <= 1.5;
  }

  updateSpecificCountry(): void {
    const updatedCountries = this.specificSendingCountries
      .filter((con) => con.isChecked)
      .map((con) => {
        const sendCountryId = this.sendCountries.find(
          (c) => c.CountryName === con.SendCountryName
        )?.CountryID;
        const sendCurrencyId = this.sendCurrencies.find(
          (c) => c.CurrencyShortName === con.SendCurrencyName
        )?.Currency_Id;

        return {
          ID: con?.ID,
          PayInCountryId: sendCountryId,
          PayInCurrencyId: sendCurrencyId,
          MarginPercent: Number(con?.MarginPercent),
          Max_AgentMargin: Number(con?.MaximumAgentMargin),
          Min_AgentMargin: Number(con?.MinimumAgentMargin),
          Max_LimitForPremiumRate: Number(con?.MaximumPremiumRateLimit),
          Active: true,
          EnteredBy: '1001000',
        };
      });
    if (updatedCountries?.length) {
      this.specificCountryService
        .updateSpecificSendingCountry(
          getCacheData('admin_token'),
          getCacheData('username'),
          updatedCountries
        )
        .subscribe({
          next: (res: any) => {
            this.modalService.hide();
            alert('Exchange Margin updated successfully.');
            this.getAllSendingCountries();
            this.getMarginSendingCountry();
          },
          error: (err) => {
            this.modalService.hide();
            alert(err?.error?.message || 'Something went wrong');
          },
          complete: () => {
            this.modalService.hide();
          },
        });
    }
  }

  allowOneDecimal(event: Event): void {
    const inputElement = event.target as HTMLInputElement;
    const inputValue = inputElement.value;

    // Remove multiple decimals and keep only the first one
    const sanitizedValue = inputValue.replace(/(\.\d*)\./, '$1');

    // Update the input value
    inputElement.value = sanitizedValue;
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
