import { Component, TemplateRef, ViewChild } from '@angular/core';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { MarginsSpecificSendingCountryService } from '../services/margins-specific-sending-country.service';
import { getCacheData } from '../utils/helper';
import { MarginsSpecificSendingReceivingCountryService } from '../services/margins-specific-sending-receiving-country.service';

@Component({
  selector: 'app-margins-specific-sending-receiving-country',
  templateUrl: './margins-specific-sending-receiving-country.component.html',
  styleUrls: ['./margins-specific-sending-receiving-country.component.css'],
})
export class MarginsSpecificSendingReceivingCountryComponent {
  @ViewChild('manageSpecificSendingCountry') manageSpecificSendingCountryModal;
  bsModalRef: BsModalRef;

  sendingCountries: any = [];
  selectedCountry: any = 'All';

  type: any = 'add';

  receivingCountries: any = [];
  selectedReceivingCountry: any = 'All';
  sendCurrencies: any = [];
  receivingCurrencies: any = [];
  modeofPayments: any = [];

  specificSendingCountries: any = [];

  manageSpecificSendingReceivingCountryForm = new FormGroup({
    ID: new FormControl(),
    PayInCountryId: new FormControl('None', Validators.required),
    PayInCurrencyId: new FormControl('', Validators.required),
    MarginPercent: new FormControl('', Validators.required),
    Max_AgentMargin: new FormControl('', Validators.required),
    Min_AgentMargin: new FormControl('', Validators.required),
    Max_LimitForPremiumRate: new FormControl('', Validators.required),
    ExchangeSetName: new FormControl('', Validators.required),
    PayOutCountryId: new FormControl('', Validators.required),
    ModeOfPayemntId: new FormControl('', Validators.required),
    PayOutCurrencyId: new FormControl('', Validators.required),
  });

  constructor(
    private specificCountryService: MarginsSpecificSendingCountryService,
    private specificRecevingCountryService: MarginsSpecificSendingReceivingCountryService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getAllSendingCountries();
    this.getMarginSendingReceivingCountry();
    this.getMarginReceivingCountry();
  }

  getAllSendingCountries(payInCountryId = 0, payOutCountryId = 0) {
    this.specificRecevingCountryService
      .getAllSendingReceivingCountries(
        getCacheData('admin_token'),
        getCacheData('username'),
        payInCountryId,
        payOutCountryId
      )
      .subscribe({
        next: (res: any) => {
          this.specificSendingCountries = res;

          this.specificSendingCountries = this.specificSendingCountries.map(
            (obj) => {
              const updatedObj = {};
              Object.keys(obj).forEach((key) => {
                const newKey = key.replace(/\s+/g, '_'); // Replace spaces with underscores
                updatedObj[newKey] = obj[key];
              });
              return updatedObj;
            }
          );
          for (
            let index = 0;
            index < this.specificSendingCountries.length;
            index++
          ) {
            const element = this.specificSendingCountries[index];
            if (
              element?.Margin_Percent !== null &&
              element?.Margin_Percent !== undefined
            ) {
              element.Margin_Percent = this.formatWithFourDecimalPlaces(
                element.Margin_Percent
              );
            }
            if (
              element?.Maximum_Agent_Margin !== null &&
              element?.Maximum_Agent_Margin !== undefined
            ) {
              element.Maximum_Agent_Margin = this.formatWithFourDecimalPlaces(
                element.Maximum_Agent_Margin
              );
            }
            if (
              element?.Minimum_Agent_Margin !== null &&
              element?.Minimum_Agent_Margin !== undefined
            ) {
              element.Minimum_Agent_Margin = this.formatWithFourDecimalPlaces(
                element.Minimum_Agent_Margin
              );
            }
            if (
              element?.Maximum_Premium_Rate_Limit !== null &&
              element?.Maximum_Premium_Rate_Limit !== undefined
            ) {
              element.Maximum_Premium_Rate_Limit =
                this.formatWithFourDecimalPlaces(
                  element.Maximum_Premium_Rate_Limit
                );
            }
          }
        },
        error: (err) => {
          this.specificSendingCountries = [];
        },
        complete: () => {},
      });
  }

  formatWithFourDecimalPlaces(value: number | string): string {
    if (value === null || value === undefined) {
      return '';
    }

    const stringValue = value.toString();
    const [integerPart, fractionalPart] = stringValue.split('.');

    if (fractionalPart) {
      // Pad with zeros to ensure 4 decimal places
      return `${integerPart}.${fractionalPart.padEnd(4, '0').slice(0, 4)}`;
    } else {
      // If no decimal places, add two zeros
      return `${integerPart}.0000`;
    }
  }

  getMarginSendingReceivingCountry() {
    this.specificRecevingCountryService
      .getMarginSendingReceivingCountry(
        getCacheData('admin_token'),
        getCacheData('username')
      )
      .subscribe({
        next: (res: any) => {
          this.sendingCountries = res;
          this.getMarginSendCurrency(res[0]?.PayInCountryId);
          this.getMarginReceivingCountry(res[0]?.PayInCountryId);
        },
        error: (err) => {
          this.sendingCountries = [];
        },
        complete: () => {},
      });
  }

  getMarginReceivingCountry(countryId: any = 0) {
    this.specificRecevingCountryService
      .getMarginReceivingCountry(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.receivingCountries = res;
          this.getMarginReceiveCurrency(res?.[0]?.PayOutCountryId);
          this.getMarginModeofPayment(res?.[0]?.PayOutCountryId);
        },
        error: (err) => {
          this.receivingCountries = [];
        },
        complete: () => {},
      });
  }

  getMarginSendCurrency(countryId = 0) {
    this.specificRecevingCountryService
      .getMarginSendCurrency(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.sendCurrencies = res;
        },
        error: (err) => {
          this.sendCurrencies = [];
        },
        complete: () => {},
      });
  }

  getMarginReceiveCurrency(countryId = 0) {
    this.specificRecevingCountryService
      .getMarginReceiveCurrency(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.receivingCurrencies = res;
        },
        error: (err) => {
          this.receivingCurrencies = [];
        },
        complete: () => {},
      });
  }

  getMarginModeofPayment(countryId = 0) {
    this.specificRecevingCountryService
      .getMarginModeofPayment(
        getCacheData('admin_token'),
        getCacheData('username'),
        countryId
      )
      .subscribe({
        next: (res: any) => {
          this.modeofPayments = res;
        },
        error: (err) => {
          this.modeofPayments = [];
        },
        complete: () => {},
      });
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openSpecificCountryModal(type: any) {
    this.type = type;

    if (type === 'add') {
      this.manageSpecificSendingReceivingCountryForm.reset({
        PayInCountryId: 'None',
        PayInCurrencyId: 'None',
        ExchangeSetName: '',
        PayOutCountryId: 'None',
        PayOutCurrencyId: 'None',
        ModeOfPayemntId: 'None',
        MarginPercent: '',
        Max_AgentMargin: '',
        Max_LimitForPremiumRate: '',
        Min_AgentMargin: '',
      });
      if (this.f.PayInCountryId === 'None') {
        this.sendCurrencies = [];
      }
    }

    this.openModal(this.manageSpecificSendingCountryModal);
  }

  searchSpecficCountry() {
    let sendingCountryId: number | null = 0;

    let receivingCountryId: number | null = 0;

    if (this.selectedCountry !== 'All') {
      sendingCountryId = this.selectedCountry;
    }

    if (this.selectedReceivingCountry !== 'All') {
      receivingCountryId = this.selectedReceivingCountry;
    }

    this.getAllSendingCountries(sendingCountryId, receivingCountryId);
  }

  get f() {
    return this.manageSpecificSendingReceivingCountryForm.value;
  }

  addSpecificCountry() {
    if (!this.manageSpecificSendingReceivingCountryForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    if (this.f.PayInCountryId == 'None') {
      alert('Please Select Send Country!');
      return;
    }

    if (this.f.PayInCurrencyId == 'None') {
      alert('Please Select Send Currency!');
      return;
    }

    if (this.f.PayOutCountryId == 'None') {
      alert('Please Select Receive Country!');
      return;
    }

    if (this.f.PayOutCurrencyId == 'None') {
      alert('Please Select Receive Currency!');
      return;
    }

    if (this.f.ModeOfPayemntId == 'None') {
      alert('Please Select Mode of Payment!');
      return;
    }

    if (
      Number(this.f.MarginPercent) <= 0.1 ||
      Number(this.f.MarginPercent) >= 1.5
    ) {
      alert('Margin Percent should not be greater than 100%!');
      return;
    }

    if (this.f.Max_AgentMargin <= this.f.Min_AgentMargin) {
      alert(
        'Maximum Agent Margin Should be greater than Minimum Agent Margin!'
      );
      return;
    }

    if (this.f.MarginPercent < this.f.Max_LimitForPremiumRate) {
      alert('Maximum Premium Rate Limit Should be less than Margin Percent!');
      return;
    }

    let payload;
    payload = {
      PayInCountryId: this.f.PayInCountryId,
      PayInCurrencyId: this.f.PayInCurrencyId,
      PayOutCountryId: this.f.PayOutCountryId,
      ModeOfPayemntId: this.f.ModeOfPayemntId,
      PayOutCurrencyId: this.f.PayOutCurrencyId,
      MarginPercent: Number(this.f.MarginPercent),
      Max_AgentMargin: Number(this.f.Max_AgentMargin),
      Min_AgentMargin: Number(this.f.Min_AgentMargin),
      Max_LimitForPremiumRate: Number(this.f.Max_LimitForPremiumRate),
      Active: true,
      EnteredBy: 'none',
      ExchangeSetName: this.f.ExchangeSetName,
    };

    if (this.type === 'edit') {
      payload = [
        {
          ID: this.f.ID,
          ExchangeSetName: this.f.ExchangeSetName,
          PayInCountryId: this.f.PayInCountryId,
          EnteredBy: 'none',
          PayInCurrencyId: this.f.PayInCurrencyId,
          PayOutCountryId: this.f.PayOutCountryId,
          PayOutCurrencyId: this.f.PayOutCurrencyId,
          ModeOfPayemntId: this.f.ModeOfPayemntId,
          MarginPercent: Number(this.f.MarginPercent),
          Max_AgentMargin: Number(this.f.Max_AgentMargin),
          Min_AgentMargin: Number(this.f.Min_AgentMargin),
          Max_LimitForPremiumRate: Number(this.f.Max_LimitForPremiumRate),
          Active: true,
        },
      ];
    }

    const serviceMethod =
      this.type === 'add'
        ? 'addSpecificSendingReceivingCountry'
        : 'updateSpecificSendingReceivingCountry';

    this.specificRecevingCountryService[serviceMethod](
      getCacheData('admin_token'),
      getCacheData('username'),
      payload
    ).subscribe({
      next: (res: any) => {
        this.modalService.hide();
        this.getAllSendingCountries();
        this.getMarginSendingReceivingCountry();
      },
      error: (err) => {
        this.modalService.hide();
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  onSendCountryChange(countryId) {
    if (countryId != 'None') {
      this.getMarginSendCurrency(countryId);
      this.getMarginReceivingCountry(countryId);
    }
  }

  onReceiveCountryChange(countryId) {
    if (countryId != 'None') {
      this.getMarginReceiveCurrency(countryId);
      this.getMarginModeofPayment(countryId);
    }
  }

  onReceiveCurrencyChange(countryId) {
    if (countryId !== 'None') {
      // Ensure data arrays are initialized before accessing them
      if (
        this.sendingCountries &&
        this.sendCurrencies &&
        this.receivingCountries &&
        this.receivingCurrencies
      ) {
        const sendCountry = this.sendingCountries.find(
          (s) =>
            s.PayInCountryId ==
            this.manageSpecificSendingReceivingCountryForm.get('PayInCountryId')
              .value
        ).CountryName;

        const sendCurrency = this.sendCurrencies.find(
          (s) =>
            s.Currency_Id ==
            this.manageSpecificSendingReceivingCountryForm.get(
              'PayInCurrencyId'
            ).value
        ).CurrencyShortName;

        const receiveCountry = this.receivingCountries.find(
          (s) =>
            s.PayOutCountryId ==
            this.manageSpecificSendingReceivingCountryForm.get(
              'PayOutCountryId'
            ).value
        ).CountryName;

        const receiveCurrency = this.receivingCurrencies.find(
          (s) =>
            s.CountryID ==
            this.manageSpecificSendingReceivingCountryForm.get(
              'PayOutCurrencyId'
            ).value
        ).CurrencyShortName;

        this.manageSpecificSendingReceivingCountryForm.patchValue({
          ExchangeSetName: `${sendCountry} (${sendCurrency}) - ${receiveCountry} (${receiveCurrency})`,
        });
      }
    }
  }

  isValidMarginPercent(percent: number): boolean {
    return Number(percent) >= 0.1 && Number(percent) <= 1.5;
  }

  isValidMaxAgentMargin(maxMargin: number, minMargin: number): boolean {
    return Number(maxMargin) > Number(minMargin);
  }

  isValidPremiumLimit(percent: number, limit: number): boolean {
    return Number(limit) < Number(percent);
  }

  updateSpecificCountry(): void {
    const updatedCountries = this.specificSendingCountries
      .filter((con) => con.isChecked)
      .map((con) => {
        const receiveCountryId = this.receivingCountries.find(
          (c) => (c.CountryName = con.Receive_Country_Name)
        ).PayOutCountryId;
        const receiveCurrencyId = this.receivingCurrencies.find(
          (c) => (c.CurrencyShortName = con.Receive_Currency_Name)
        ).Currency_Id;

        return {
          ID: con?.ID,
          PayInCountryId: con?.SendCountryID,
          PayInCurrencyId: con?.SendCurrencyID,
          PayOutCountryId: receiveCountryId,
          ModeOfPayemntId: con?.ModeofPayment_id,
          PayOutCurrencyId: receiveCurrencyId,
          MarginPercent: Number(con?.Margin_Percent),
          Max_AgentMargin: Number(con?.Maximum_Agent_Margin),
          Min_AgentMargin: Number(con?.Minimum_Agent_Margin),
          Max_LimitForPremiumRate: Number(con?.Maximum_Premium_Rate_Limit),
          Active: true,
          EnteredBy: 'none',
          ExchangeSetName: con?.Exchange_Set_Name,
        };
      });
    if (updatedCountries?.length) {
      this.specificRecevingCountryService
        .updateSpecificSendingReceivingCountry(
          getCacheData('admin_token'),
          getCacheData('username'),
          updatedCountries
        )
        .subscribe({
          next: (res: any) => {
            this.modalService.hide();
            this.getAllSendingCountries();
            this.getMarginSendingReceivingCountry();
          },
          error: (err) => {
            this.modalService.hide();
            alert(err?.error?.message || 'Something went wrong');
          },
          complete: () => {
            this.modalService.hide();
          },
        });
    }
  }

  editSpecificSendingCountry(country: any) {
    const receiveCountryId = this.receivingCountries.find(
      (c) => (c.CountryName = country.Receive_Country_Name)
    ).PayOutCountryId;
    const receiveCurrencyId = this.receivingCurrencies.find(
      (c) => (c.CurrencyShortName = country.Receive_Currency_Name)
    ).Currency_Id;
    this.manageSpecificSendingReceivingCountryForm.patchValue({
      ID: country?.ID,
      PayInCountryId: country?.SendCountryID,
      PayInCurrencyId: country?.SendCurrencyID,
      PayOutCountryId: receiveCountryId,
      PayOutCurrencyId: receiveCurrencyId,
      ModeOfPayemntId: country?.ModeofPayment_id,
      ExchangeSetName: country?.Exchange_Set_Name,
      MarginPercent: country?.Margin_Percent,
      Max_AgentMargin: country?.Maximum_Agent_Margin,
      Min_AgentMargin: country?.Minimum_Agent_Margin,
      Max_LimitForPremiumRate: country?.Maximum_Premium_Rate_Limit,
    });
    this.openSpecificCountryModal('edit');
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
