import { Component, TemplateRef, ViewChild } from '@angular/core';
import { RelationshipManagementService } from '../services/relationship-management.service';
import { FormGroup, FormControl, Validators } from '@angular/forms';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';

@Component({
  selector: 'app-relationship-management',
  templateUrl: './relationship-management.component.html',
  styleUrls: ['./relationship-management.component.css'],
})
export class RelationshipManagementComponent {
  @ViewChild('relationManagement') relationManagementModal;
  bsModalRef: BsModalRef;

  relationShipManagement: any = [];
  allRelationShipManagement: any = [];
  selectedRelation: any = '-1';
  type: any = 'add';

  relationManagementForm = new FormGroup({
    RelationName: new FormControl('', Validators.required),
    IsActive: new FormControl(false),
  });

  constructor(
    private relationService: RelationshipManagementService,
    private modalService: BsModalService
  ) {}

  ngOnInit(): void {
    this.getRelationList();
  }

  getRelationList() {
    this.relationService.getRelationList().subscribe({
      next: (res: any) => {
        this.relationShipManagement = res;
        this.allRelationShipManagement = res;
      },
      error: (err) => {
        this.relationShipManagement = [];
      },
      complete: () => {},
    });
  }

  addRelationManagement() {
    if (!this.relationManagementForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }

    let payload;
    payload = {
      RelationName: this.f.RelationName,
      IsActive: this.f.IsActive,
      UserID: 0,
    };

    this.relationService.addRelationManagement(payload).subscribe({
      next: (res: any) => {
        this.getRelationList();
        this.modalService.hide();
      },
      error: (err) => {
        this.modalService.hide();
        alert(err?.error?.message || 'Something went wrong');
      },
      complete: () => {
        this.modalService.hide();
      },
    });
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openRelationManagementModal(type: any) {
    if (type == 'add') {
      this.relationManagementForm.reset();
    }
    this.type = type;
    this.openModal(this.relationManagementModal);
  }

  get f() {
    return this.relationManagementForm.value;
  }

  reset() {
    this.relationManagementForm.reset();
  }

  filter() {
    if (this.selectedRelation) {
      this.relationService
        .getRelationListByRelationName(this.selectedRelation)
        .subscribe({
          next: (res: any) => {
            this.relationShipManagement = res;
          },
          error: (err) => {
            this.relationShipManagement = [];
          },
          complete: () => {},
        });
    }
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
