import { Component, ElementRef, ViewChild } from '@angular/core';
import { SearchTransactionsService } from '../services/search-transactions.service';
import * as XLSX from 'xlsx';

@Component({
  selector: 'app-search-transactions',
  templateUrl: './search-transactions.component.html',
  styleUrls: ['./search-transactions.component.css'],
})
export class SearchTransactionsComponent {
  @ViewChild('transactionTable') transactionTable: ElementRef;

  lcCurrencyList: any = [];
  selectedLcCurrency: any;
  sendAgentList: any = [];
  selectedSendAgent: any;
  fcCurrencyList: any = [];
  selectedFcCurrency: any;
  receiveAgentList: any = [];
  selectedReceiveAgent: any;
  customerFullName: any = '';
  beneficiaryFullName: any = '';
  fromAmount: any = '';
  toAmount: any = '';
  isUsd: any = false;
  startDate: any = '';
  endDate: any = '';
  allTransactionDetails: any = [];
  reportTypes: any = [
    {
      id: 1,
      type: 'Normal',
    },
    {
      id: 2,
      type: 'CCY Wise',
    },
  ];
  selectedReportType: any = 1;

  constructor(private searchTransactionSvc: SearchTransactionsService) {}

  ngOnInit(): void {
    this.getLcCurrencyList();
    this.getFcCurrencyList();
    this.getReceivingAgentList(-1);
  }

  getLcCurrencyList() {
    this.searchTransactionSvc.getLcCurrencyList().subscribe({
      next: (res: any) => {
        this.lcCurrencyList = res;
        this.selectedLcCurrency = res?.[0]?.Currency_Id;
        this.getSendAgentList(this.selectedLcCurrency);
      },
      error: (err) => {
        this.lcCurrencyList = [];
      },
      complete: () => {},
    });
  }

  onLcCurrencyChange(currencyId: any) {
    if (currencyId != -1) {
      this.getSendAgentList(currencyId);
    }
  }

  getSendAgentList(currencyId: any = 0) {
    this.searchTransactionSvc.getSendAgentList(currencyId).subscribe({
      next: (res: any) => {
        this.sendAgentList = res;
        if (currencyId > 0) {
          this.selectedSendAgent = res?.[0]?.agentid;
        }
      },
      error: (err) => {
        this.sendAgentList = [];
      },
      complete: () => {},
    });
  }

  getFcCurrencyList() {
    this.searchTransactionSvc.getFcCurrencyList().subscribe({
      next: (res: any) => {
        this.fcCurrencyList = res;
        this.selectedFcCurrency = res?.[0]?.Currency_Id;
      },
      error: (err) => {
        this.fcCurrencyList = [];
      },
      complete: () => {},
    });
  }

  onFcCurrencyChange(currencyId: any) {
    if (currencyId != -1) {
      this.getReceivingAgentList(currencyId);
    }
  }

  getReceivingAgentList(currencyId: any = -1) {
    this.searchTransactionSvc.getReceivingAgentList(currencyId).subscribe({
      next: (res: any) => {
        this.receiveAgentList = res;
        if (currencyId == -1) {
          this.selectedReceiveAgent = res?.[0]?.agentid;
        }
      },
      error: (err) => {
        this.receiveAgentList = [];
      },
      complete: () => {},
    });
  }

  searchTransaction() {
    if (!this.startDate && !this.endDate) {
      alert('Please Select Start & End Date!');
      return;
    }

    let payload: any = {};

    payload = {
      ReportType: this.selectedReportType,
      LcCurrencyId: this.selectedLcCurrency,
      SendAgentId: this.selectedSendAgent ? this.selectedSendAgent : -1,
      FcCurrencyId: this.selectedFcCurrency,
      ReceiveAgentId: this.selectedReceiveAgent
        ? this.selectedReceiveAgent
        : -1,
      CustFullname: this.customerFullName ? this.customerFullName : '',
      Benfullname: this.beneficiaryFullName ? this.beneficiaryFullName : '',
      Fromamount: this.fromAmount ? this.fromAmount : '',
      ToAmount: this.toAmount ? this.toAmount : '',
      IsAmountinUSD: this.isUsd ? '1' : '0',
      StartDate: this.startDate,
      EndDate: this.endDate,
    };
    this.searchTransactionSvc.searchTransactionDetail(payload).subscribe({
      next: (res: any) => {
        this.allTransactionDetails = res;
      },
      error: () => {
        this.allTransactionDetails = [];
      },
      complete: () => {},
    });
  }

  exportToExcel() {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.transactionTable.nativeElement
    );

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `Search_Transactions${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }

  getTotal(property: string): number {
    const sanitizedProperty = property.replace(/\s+/g, '');

    if (this.allTransactionDetails?.[0]?.hasOwnProperty(sanitizedProperty)) {
      return this.allTransactionDetails[0][sanitizedProperty];
    }

    return 0;
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
