import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { Observable } from 'rxjs';
import { environment } from 'src/environments/environment';
import { getCacheData } from '../utils/helper';

@Injectable({
  providedIn: 'root',
})
export class AgentWiseRatesService {
  url = environment.apiUrl;

  constructor(private router: Router, private http: HttpClient) {}

  getAllAgentWiseRates(
    token: any,
    username,
    sendingcountryId: any = 0,
    sendingagentId: any = 0,
    receivingcountryId: any = 0,
    receivingagentId: any = 0
  ) {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/agent-wise-rate/exchange-rate-bank-rate-agentWise?sendingcountryId=${sendingcountryId}&sendingagentId=${sendingagentId}&receivingcountryId=${receivingcountryId}&receivingagentId=${receivingagentId}`,
      {
        headers: headers,
      }
    );
  }

  getSendingCountry(token: any, username, userId: any = 0): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/agent-wise-rate/sending-country-dropDown
     `,
      {
        headers: headers,
      }
    );
  }

  getSendingAgent(
    token: any,
    username,
    type: any = 0,
    sendingcountryId: any = 0
  ): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/agent-wise-rate/sending-agent-dropdown?type=${type}&sendingcountryId=0
     `,
      {
        headers: headers,
      }
    );
  }

  addAgentWiseExchangeRates(
    token: any,
    username,
    payload: any
  ): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.post(
      `${environment.apiUrl}/v1/api/master/agent-wise-rate/exchange-rate-bank-rate-agentWise
     `,
      payload,
      {
        headers: headers,
      }
    );
  }

  updateAgentWiseExchangeRates(
    token: any,
    username,
    payload: any
  ): Observable<any> {
    const headers = new HttpHeaders({
      Authorization: token,
      Accept: 'application/json',
      username: username,
    });
    return this.http.patch(
      `${environment.apiUrl}/v1/api/master/agent-wise-rate/exchange-rate-bank-rate-agentWise
     `,
      payload,
      {
        headers: headers,
      }
    );
  }

  getModeOfPayment(
    targetCountryId: any = 0,
    receiveAgentId: any = 0
  ): Observable<any> {
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/agent-wise-rate/mode-of-payment?TargetCountryID=${targetCountryId}&ReceAgentID=${receiveAgentId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getSettlementCurrency(agentId: any = 0): Observable<any> {
    return this.http.get(
      `${environment.apiUrl}/v1/api/master/agent-wise-rate/settlement-currency?AgentID=${agentId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
}
